/****************************************************************/
/* SETUP                                                        */
/****************************************************************/
/* This driver has been tested with the ADS8330 EVM             */
/* (preliminary version) and the TMS320C6713 DSK using CCS v3.1 */
/*                                                              */
/* For hardware settings refer to the html file in the project  */
/* directory                                                    */
/****************************************************************/


#include <stdlib.h>
#include "dc_conf.h"
#include "t8330_fn.h"
#include "tidc_api.h"

#define BUFF_SZ (2048)

unsigned int Buffer1[BUFF_SZ];
unsigned int Buffer2[BUFF_SZ];
long lSample;
volatile unsigned short usChannel0, usChannel1;

volatile int iSemaphore = 0;

void TransferFinished(void* pDac);

void main (void)
{
	unsigned int uiParam;

	/* connect MCBSP to daughterboard connector		            */
	if ((&Ads8330_1)->serial->port == 0)
		*(unsigned volatile char *)0x90080006 |= 0x01;
	else if ((&Ads8330_1)->serial->port == 1)
		*(unsigned volatile char *)0x90080006 |= 0x02;
    
	/* allocate resources and put converter in power down mode  */
	if (dc_configure(&Ads8330_1) != TIDC_NO_ERR) {
		dc_close(&Ads8330_1);	
		abort();
	}
	IRQ_globalEnable();

    /* read buffers                                             */
    (void)dc_readblock(&Ads8330_1, Buffer1, BUFF_SZ, TransferFinished);
    iSemaphore++;

    (void)dc_readblock(&Ads8330_1, Buffer2, BUFF_SZ, TransferFinished);
    iSemaphore++;
	while(iSemaphore > 0);

	uiParam = ADS8330_PD_DEEP;
	dc_control(&Ads8330_1, ADS8330_CMD_POWERDOWN, (void*)&uiParam);
	dc_control(&Ads8330_1, ADS8330_CMD_WAKEUP, 0);

    (void)dc_readblock(&Ads8330_1, Buffer1, BUFF_SZ, TransferFinished);
    iSemaphore++;
	while(iSemaphore > 0);

    /* read single sample                                       */
    (void)dc_readsample(&Ads8330_1, &lSample);
	if (lSample & 0x0000000008) {
		usChannel0 = lSample >> 24;
		usChannel1 = (lSample >> 4) & 0xFFFF;
	} else {
		usChannel1 = lSample >> 24;
		usChannel0 = (lSample >> 4) & 0xFFFF;
	}
    (void)dc_readblock(&Ads8330_1, Buffer1, BUFF_SZ, TransferFinished);
    iSemaphore++;
	while(iSemaphore > 0);

	/* free all by the software allocate resources              */
	dc_close(&Ads8330_1);

	/* allocate resources and put converter in power down mode  */
	if (dc_configure(&Ads8330_1) != TIDC_NO_ERR) {
		dc_close(&Ads8330_1);	
		abort();
	}

    (void)dc_readblock(&Ads8330_1, Buffer2, BUFF_SZ, TransferFinished);
    iSemaphore++;
	while(iSemaphore > 0);

	/* free all resources allocate by the software              */
	dc_close(&Ads8330_1);

    /* wait here forever ...                                    */
    while(1);
}

void TransferFinished(void* pDac)
{
	iSemaphore--;
}
