/*{{TIDC_Wizard Auto Code Start                                 */
/****************************************************************/
/* don't change anything here until you know what you are doing!*/
/* the plug-in will change all text between the code marks      */
/* without user interaction!                                    */
/*                                                              */
/****************************************************************/

/****************************************************************/
/* software generated by Data Converter Plug-in (DCP)           */
/* based on <C:\CCSTUD~1.1\PLUGINS\AAP_DEV\c6000\d8328_62.c> */
/****************************************************************/
/****************************************************************/
/* ADS8330 driver version 1.0 for the TMS320C6x1x DSPs          */
/* Copyright  2002-2007 Texas Instruments Incorporated         */
/* All Rights Reserved                                          */
/*--------------------------------------------------------------*/
/** \file
 * The dispatcher function DCPDISP_dispatchEdmaIsr() defined in
 * tidc_api.c must be hooked to the EDMA hardware interrupt
 * (default HWI_INT8 in the configuration database (cdb-file)).
 *
 * For assistance read the application note "Getting started
 * with the Data Converter Plug-In" (Literature number slaa210)
 * 
 * NOTES:
 * Channel Sorting
 *   Channel sorting is not supported by the software.
 *   Because of speed issues the data samples are "read as they come".
 *   Thus, it is not determinable whether samples of channel 0 or
 *   channel 1 are located at odd/even memory locations in the buffer.
 *   The application layer has to check for the TAG flag (bit 3)
 *   to determine the channel the data sample comes from.
 * Reading a Single Sample
 *   The lower 16bit of pass-by-reference value "lData" contain
 *   a sample of channel 0, the upper 16bit a sample of channel 1.
 *   The TAG bit is removed.
 * Shift/Mask of Data Samples
 *   The data samples are not shifted or masked.
 *   To get the sample values a right shift by 4 is needed.
 *  Serial Port Speed
 *   The serial port clock should be greater than 32MHz to ensure
 *   data are read before the next conversion starts.
 * Serial Port Speed Issue
 *   During development of this software communication problems
 *   occured at frequencies near the converter serial port limit.
 * Converter Read Mode
 *   This software read the data from the converter
 *   in "Read-while-Converting" mode.
 * Conversion Clock
 *   The converter uses the internal oscillator as conversion
 *   clock only (no serial port clock support).
 * Pin 10 Function
 *   Pin 10 is set to EOC functionality always.                 */
/*--------------------------------------------------------------*/
/* $Revision: 1 $                      $JustDate:  2/23/07 $    */
/* $Author: A0797073 $                                          */
/****************************************************************/

#undef _INLINE

#include <string.h>
#include "dc_conf.h"
#include "t8330_fn.h"

/*****************************************************************/
/* static function prototypes                                    */
/*****************************************************************/
static TTIDCSTATUS ConfigVerify(const TADS8330 *pAdc);
static TTIDCSTATUS OpenEdma(TADS8330 *pAdc);
static TTIDCSTATUS InitExtInt(const TADS8330 *pAdc);
static TTIDCSTATUS InitMcBsp(TADS8330 *pAdc);
static TTIDCSTATUS InitTimer(TADS8330 *pAdc);
static TTIDCSTATUS DCP_ADS8330_Isr(void *pDC);

/*****************************************************************/
/* define and initialize data converter objects                  */
/*****************************************************************/
#define ADS8330_STAT_NOTINIT   (-1)
#define ADS8330_STAT_READY     ( 0)
#define ADS8330_STAT_RUN       ( 1)
#define ADS8330_STAT_BUSY      ( 2)

TADS8330 Ads8330_1 = {
    &ADS8330_configure,
    &ADS8330_control,
    &ADS8330_readsample,
    &ADS8330_writesample,
    &ADS8330_readblock,
    &ADS8330_writeblock,
    &ADS8330_close,
    0, 0, 3,            /* reserved                              */
    &serial0,                           /* used McBSP port          */
    0,                                  /* callback func (current)  */
    0,                                  /* callback func (pending)  */
    ADC1_CREG,                           /* Configuration register   */
    ADC1_INTNUM,                        /* DSP int used by Pin10    */
    0,                                  /* Timer handle             */
    ADC1_TIMNUM,                        /* used Timer, if any       */
    ADC1_TIMDIV,                        /* Divider for Timer clock  */
    0, 0, 0, 0, 0,                      /* EDMA handles             */
    ADS8330_STAT_NOTINIT                /* transfer flag                     */
};

DCP_SERIAL serial0 =
{
   ADC1_SERPORT,        /* Serial port number                    */
   0,                   /* interrupt used by DC driver           */
   0                    /* McBSP handle                          */
};

/*****************************************************************/
/* ADS8330_configure()                                           */
/**Operation:
 *    - Initializes the McBSP
 *    - Initializes the EDMA
 *
 * Parameters:
 *    - Data converter object
 *
 * Return value:
 *    - all from ADS8330_control()
 *    - TIDC_ERR_NOCHIPRES                                       */
/*****************************************************************/
TTIDCSTATUS ADS8330_configure(void* pDC)
{
   TADS8330* pAdc = (TADS8330*)pDC;
   TTIDCSTATUS status;
   
   /* be sure the resources are not open/init yet                */
   if (pAdc->iDriverStatus != ADS8330_STAT_NOTINIT)
      return TIDC_ERR_NOCHIPRES;

   status = InitMcBsp(pAdc);
   if (status != TIDC_NO_ERR) {
      (void)ADS8330_close(pDC);
      return status;
   }
   status = InitExtInt(pAdc);
   if (status != TIDC_NO_ERR) {
      (void)ADS8330_close(pDC);
      return status;
   }
   status = ConfigVerify(pAdc);
   if (status != TIDC_NO_ERR) {
      (void)ADS8330_close(pDC);
      return status;
   }
   status = OpenEdma(pAdc);
   if (status != TIDC_NO_ERR) {
      (void)ADS8330_close(pDC);
      return status;
   }
   status = InitTimer(pAdc);
   if (status != TIDC_NO_ERR) {
      (void)ADS8330_close(pDC);
      return status;
   }
   status = DCPDISP_registerIsr(pDC, DCP_ADS8330_Isr, pAdc->serial->intnum);
   if (status != TIDC_NO_ERR) {
      (void)ADS8330_close(pDC);
      return status;
   }
   
   /* let the other functions know the resources are open/init   */
   pAdc->iDriverStatus = ADS8330_STAT_READY;
   
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* ADS8330_control()                                             */
/**Operation:
 *    - Controls the converter
 *    - the following commands (+) and parameter (*) are supported:
 *      + ADS8330_CMD_POWERDOWN
 *        * ADS8330_PD_DEEP
 *        * ADS8330_PD_NAP
 *        * ADS8330_PD_AUTO
 *      + ADS8330_CMD_WAKEUP
 *
 * Parameters:
 *    - Data converter object
 *    - command
 *    - parameter for command (if any)
 *
 * Return value:
 *    - TIDC_NO_ERR
 *    - TIDC_ERR_BADARGS
 *    - TIDC_ERR_NODEVICE
 *    - TIDC_ERR_XFERPROG                                        */
/*****************************************************************/
TTIDCSTATUS ADS8330_control(void *pDC, int iCmd, void *pValue)
{
   TADS8330* pAdc = (TADS8330*)pDC;
   unsigned int uiParam = *(unsigned int*)pValue;
   volatile unsigned int uiData;
   
   if (pAdc->iDriverStatus == ADS8330_STAT_NOTINIT)
      return TIDC_ERR_NODEVICE;
   if (pAdc->iDriverStatus != ADS8330_STAT_READY)
      return TIDC_ERR_XFERPROG;

   if (iCmd == ADS8330_CMD_POWERDOWN) {
      uiData = ADS8330_WRCFR | (pAdc->creg.value << 4);
      if (uiParam == ADS8330_PD_DEEP) {
         uiData = uiData & ~ADS8330_PD_DEEP;
      } else if (uiParam == ADS8330_PD_NAP) {
         uiData = uiData & ~ADS8330_PD_NAP;
      } else if (uiParam == ADS8330_PD_AUTO) {
         uiData = uiData & ~ADS8330_PD_AUTO;
      } else {
         return TIDC_ERR_BADARGS;
      }
   } else if (iCmd == ADS8330_CMD_WAKEUP) {
      uiData = ADS8330_WAKEUP;
   } else {
      return TIDC_ERR_BADARGS;
   }
   MCBSP_write(pAdc->serial->hMcbsp, uiData);
   /* wait until transfer is completed                           */
   while (MCBSP_rrdy(pAdc->serial->hMcbsp) == 0);
   uiData = MCBSP_read(pAdc->serial->hMcbsp);
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* ADS8330_readsample()                                          */
/**Operation:
 *    - returns one sample for each channel in "lData"
 *    - because of speed isues this function issues a read block
 *    - lData: lower 16bit is sample of channel 0
 *             upper 16bit is sample of channel 1
 *
 * Parameters:
 *    - Data converter object
 *    - Memory location the data samples are stored to
 *
 * Return value:
 *    - TIDC_NO_ERR
 *    - TIDC_ERR_NODEVICE
 *    - TIDC_ERR_XFERPROG                                        */
/*****************************************************************/
TTIDCSTATUS ADS8330_readsample(void *pDC, long* lData)
{
   TADS8330* pAdc = (TADS8330*)pDC;
   unsigned int uiSamples[2];
   unsigned int uiChannel0, uiChannel1;
   
   if (pAdc->iDriverStatus == ADS8330_STAT_NOTINIT)
      return TIDC_ERR_NODEVICE;
   if (pAdc->iDriverStatus != ADS8330_STAT_READY)
      return TIDC_ERR_XFERPROG;

   /* block transfer here since sample-by-sample read is too slow*/
   (void)ADS8330_readblock(pDC, uiSamples, 2, 0);
   while(pAdc->iDriverStatus != ADS8330_STAT_READY);

   /* determine channel in memory location 0 and sort accordingly*/
   if (uiSamples[0] & 0x00000008) {
      uiChannel1 = uiSamples[0] >> 4;
      uiChannel0 = uiSamples[1] >> 4;
   } else {
      uiChannel0 = uiSamples[0] >> 4;
      uiChannel1 = uiSamples[1] >> 4;
   }
   /* build the final word                                       */
   *lData = (uiChannel1 << 16) | uiChannel0;
   
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* ADS8330_readblock()                                           */
/**Operation:
 *    - Reads a block of data words from converter
 *    - Samples are "read-as-they-come", which means no channel
 *      sorting takes place because of speed issues.
 *      In other words: It is not determinable if a sample of
 *      channel 0 or channel 1 is on the 1ist memory location of
 *      the buffer. The application layer needs to check the tag
 *      bit (bit 3) to determine the channel. 
 *
 * Parameters:
 *    - Data converter object
 *    - Pointer to data words field
 *    - Number of words
 *    - Address of callback function
 *
 * Return value:
 *    - TIDC_NO_ERR
 *    - TIDC_ERR_NODEVICE
 *    - TIDC_ERR_XFERPROG
 *    - TIDC_ERR_BADARGS                                         */
/*****************************************************************/
TTIDCSTATUS ADS8330_readblock(void *pDC,
                              void *pData,
                              unsigned long ulCount,
                              void (*callback) (void *))
{
   TADS8330* pAdc = (TADS8330*)pDC;
   EDMA_Config cfgEdmaRx, cfgEdmaTx;
   static unsigned int uiDataTx = ADS8330_RDDATA;
   
   /* parameter & availability check                             */
   if (pAdc->iDriverStatus == ADS8330_STAT_NOTINIT)
      return TIDC_ERR_NODEVICE;
   if (pAdc->iDriverStatus == ADS8330_STAT_BUSY)
      return TIDC_ERR_XFERPROG;
   if (ulCount < 0x00000002u || ulCount > 0x0000FFFFu)
      return TIDC_ERR_BADARGS;
   
   /* populate EDMA configuration structure                      */
   cfgEdmaTx.opt = ADS8330_EDMA_OPT_TX_VALUE;
   cfgEdmaTx.src = (unsigned int)&uiDataTx;
   cfgEdmaTx.cnt = (unsigned int)ulCount;
   cfgEdmaTx.dst = MCBSP_getXmtAddr(pAdc->serial->hMcbsp);
   cfgEdmaTx.idx = 0x00000000;
   cfgEdmaTx.rld = EDMA_getTableAddress(pAdc->hEdma_stop) & 0x0000FFFFu;
   
   cfgEdmaRx.opt = ADS8330_EDMA_OPT_RX_VALUE |
                 ((pAdc->serial->intnum & 0xF) << 16) |
                 ((pAdc->serial->intnum & 0x30) << 9);
   cfgEdmaRx.src = MCBSP_getRcvAddr(pAdc->serial->hMcbsp);
   cfgEdmaRx.cnt = (unsigned int)ulCount;
   cfgEdmaRx.dst = (unsigned int)pData;
   cfgEdmaRx.idx = 0x00000000;
   cfgEdmaRx.rld = EDMA_getTableAddress(pAdc->hEdma_stop) & 0x0000FFFFu;

   /* new transfer on progress - start new transfer              */
   if (pAdc->iDriverStatus == ADS8330_STAT_READY) {
      pAdc->iDriverStatus = ADS8330_STAT_RUN;
      pAdc->fCb           = callback;
      /* configure EDMA channel, prepare interrupt, start channel*/
      EDMA_config(pAdc->hEdmaTx, &cfgEdmaTx);
      EDMA_config(pAdc->hEdmaRx, &cfgEdmaRx);
      EDMA_intClear(pAdc->serial->intnum);
      EDMA_intEnable(pAdc->serial->intnum);
      EDMA_clearChannel(pAdc->hEdmaTx);
      EDMA_clearChannel(pAdc->hEdmaRx);
      EDMA_enableChannel(pAdc->hEdmaTx);
      EDMA_enableChannel(pAdc->hEdmaRx);
   }
   else {              /* ADS8330_STAT_RUN                       */
      /* transfer already on progress - append new transfer      */
      IRQ_globalDisable();
      pAdc->iDriverStatus = ADS8330_STAT_BUSY;
      pAdc->fCb_pend      = callback;
      EDMA_config(pAdc->hEdmaTx_pend, &cfgEdmaTx);
      EDMA_config(pAdc->hEdmaRx_pend, &cfgEdmaRx);
      EDMA_link(pAdc->hEdmaTx, pAdc->hEdmaTx_pend);
      EDMA_link(pAdc->hEdmaRx, pAdc->hEdmaRx_pend);
      IRQ_globalEnable();
   }
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* ADS8330_writesample()                                         */
/**This function is not implemented                              */
/*****************************************************************/
TTIDCSTATUS ADS8330_writesample(void *pDC, long lData)
{
   return TIDC_ERR_FXNNULL;
}

/*****************************************************************/
/* ADS8330_writeblock()                                          */
/**This function is not implemented                              */
/*****************************************************************/
TTIDCSTATUS ADS8330_writeblock(void *pDC,
                               void *pData,
                               unsigned long ulCount,
                               void (*callback) (void *))
{
   return TIDC_ERR_FXNNULL;
}

/*****************************************************************/
/* ADS8330_close()                                               */
/**Operation:
 *    - Closes all DSP resources used by this software
 *
 * Parameters:
 *    - Data converter object
 *
 * Return value:
 *    - TIDC_NO_ERR                                              */
/*****************************************************************/
TTIDCSTATUS ADS8330_close(void *pDC)
{
   TADS8330* pAdc = (TADS8330*)pDC;
   
   /* parameter & availability check                             */
   if (pAdc->iDriverStatus == ADS8330_STAT_NOTINIT) {
      return TIDC_ERR_NODEVICE;
   }
   if (pAdc->hTimer != 0) {
      TIMER_close(pAdc->hTimer);
   }
   /* close McBsp                                                */
   if (pAdc->serial->hMcbsp != 0) {
      MCBSP_reset(pAdc->serial->hMcbsp);
      MCBSP_close(pAdc->serial->hMcbsp);
   }
   /* close EDMA channel and free parameter sets                 */
   if (pAdc->hEdmaRx != 0) {
      EDMA_close(pAdc->hEdmaRx);
   }
   if (pAdc->hEdmaTx != 0) {
      EDMA_close(pAdc->hEdmaTx);
   }
   if (pAdc->hEdmaRx_pend != 0) {
      EDMA_freeTable(pAdc->hEdmaRx_pend);
   }
   if (pAdc->hEdmaTx_pend != 0) {
      EDMA_freeTable(pAdc->hEdmaTx_pend);
   }
   if (pAdc->hEdma_stop != 0) {
      EDMA_freeTable(pAdc->hEdma_stop);
   }
   /* unregister EDMA interrupt from dispatcher                  */
   (void)DCPDISP_unregisterIsr(pAdc->serial->intnum);   
   /* driver is now uninitalized again                           */
   pAdc->iDriverStatus = ADS8330_STAT_NOTINIT;

   return TIDC_NO_ERR;
}

/*****************************************************************/
/* DCP_ADS8330_Isr()                                             */
/**Operation:
 *    - Interrupt service routine (ISR) served at end of a block
 *      transfer
 *
 * Parameters:
 *    - None
 *
 * Return value:
 *    - None                                                     */
/*****************************************************************/
TTIDCSTATUS DCP_ADS8330_Isr(void* pDC)
{
   TADS8330* pAdc = (TADS8330*)pDC;
   
   /* call callback function if there is any                     */
   if (pAdc->fCb != (void*)0x00000000)
      pAdc->fCb(pDC);
   
   /* update status                                              */
   if (pAdc->iDriverStatus == ADS8330_STAT_BUSY) {
      pAdc->iDriverStatus = ADS8330_STAT_RUN;
      pAdc->fCb = pAdc->fCb_pend;
   }
   else { /* ADS8330_STAT_RUN                                    */
      pAdc->iDriverStatus = ADS8330_STAT_READY;
      EDMA_disableChannel(pAdc->hEdmaRx);
      EDMA_disableChannel(pAdc->hEdmaTx);
   }
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* Static functions                                              */
/*****************************************************************/
/*****************************************************************/
/* ConfigVerify()                                                */
/**Operation:
 *    - Configures the converter by writing to config register
 *    - Verifies converter availability by reading back
 *      and comparing just written config register
 *
 * Parameters:
 *    - Data converter object
 *
 * Return value:
 *    - TIDC_NO_ERR                                              */
/*****************************************************************/
static TTIDCSTATUS ConfigVerify(const TADS8330 *pAdc)
{
   volatile unsigned int uiData;
   
   /* reset converter                                            */
   MCBSP_write(pAdc->serial->hMcbsp,
               (ADS8330_WRCFR | ((pAdc->creg.value & ~0x1) << 4)));
   /* wait until transfer is completed                           */
   while (MCBSP_rrdy(pAdc->serial->hMcbsp) == 0);
   uiData = MCBSP_read(pAdc->serial->hMcbsp);

   /* write configuration register value to converter            */
   MCBSP_write(pAdc->serial->hMcbsp,
               (ADS8330_WRCFR | (pAdc->creg.value << 4)));
   /* wait until transfer is completed                           */
   while (MCBSP_rrdy(pAdc->serial->hMcbsp) == 0);
   uiData = MCBSP_read(pAdc->serial->hMcbsp);
   
   /* write "read register" command to converter                 */
   MCBSP_write(pAdc->serial->hMcbsp, ADS8330_RDCFR);
   /* wait until transfer is completed                           */
   while (MCBSP_rrdy(pAdc->serial->hMcbsp) == 0);
   /* read data and mask 4 MSB not belonging to config register  */
   uiData = (MCBSP_read(pAdc->serial->hMcbsp) >> 4) & 0x00000FFF;
   
   /* test readed data against previously written data           */
   if (uiData != pAdc->creg.value) {
      return TIDC_ERR_NODEVICE;
   }
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* InitExtInt()                                                   */
/**Operation:
 *    - No external interrupt edge initializing for C671x family
 *
 * Parameters:
 *    - Data converter object
 *
 * Return value:
 *    - TIDC_NO_ERR                                              */
/*****************************************************************/
static TTIDCSTATUS InitExtInt(const TADS8330 *pAdc)
{
#if C64_SUPPORT
   unsigned int uiEXTPOLValue, uiEPRLValue;
   /* set interrupt and EDMA event edge as requested by the user */
   uiEXTPOLValue = IRQ_FGET(EXTPOL, XIP);
   uiEPRLValue = EDMA_FGET(EPRL, EP);
   
   if (pAdc->creg.control_bit.pin10pol == 1) { /* EOC active low */
      uiEXTPOLValue = uiEXTPOLValue | (1 << (pAdc->uiIntNum-4));
      uiEPRLValue   = uiEPRLValue | (1 << pAdc->uiIntNum);      
   } else {                                   /* EOC active high */
      uiEXTPOLValue = uiEXTPOLValue | ~(1 << (pAdc->uiIntNum-4));
      uiEPRLValue   = uiEPRLValue | ~(1 << pAdc->uiIntNum);      
   }
   IRQ_FSET(EXTPOL, XIP, uiEXTPOLValue);
   EDMA_FSET(EPRL, EP, uiEPRLValue);
#endif
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* InitMcBsp()                                                   */
/**Operation:
 *    - Opens and inits the McBsp
 *
 * Parameters:
 *    - Data converter object
 *
 * Return value:
 *    - TIDC_NO_ERR
 *    - TIDC_ERR_NOCHIPRES                                       */
/*****************************************************************/
static TTIDCSTATUS InitMcBsp(TADS8330 *pAdc)
{  
   volatile unsigned int uiDelay;
   
   pAdc->serial->hMcbsp = MCBSP_open(pAdc->serial->port,
                                     MCBSP_OPEN_RESET);
   if (pAdc->serial->hMcbsp == INV) {
      pAdc->serial->hMcbsp = 0;
      return TIDC_ERR_NOCHIPRES;
   }

   /* clear content of MCBSP_config memory                     */
   memset(&pAdc->serial->sConfig, 0x0000, sizeof(MCBSP_Config));
   
   /* initialize the serial port structure                     */
   /*  Serial Port Control Register                            */
   pAdc->serial->sConfig.spcr =
      MCBSP_FMKS(SPCR, DLB, OFF)
    | MCBSP_FMKS(SPCR, RJUST, DEFAULT)
    | MCBSP_FMKS(SPCR, CLKSTP, NODELAY)
    | MCBSP_FMKS(SPCR, RINTM, DEFAULT)
    | MCBSP_FMKS(SPCR, RRST, YES)
    | MCBSP_FMKS(SPCR, FRST, YES)
    | MCBSP_FMKS(SPCR, GRST, YES)
    | MCBSP_FMKS(SPCR, XINTM, XRDY)
    | MCBSP_FMKS(SPCR, XRST, YES);

   /*  Receive Control Register                                */
   pAdc->serial->sConfig.rcr =
      MCBSP_FMKS(RCR, RFRLEN1, OF(0))
    | MCBSP_FMKS(RCR, RWDLEN1, 20BIT)
    | MCBSP_FMKS(RCR, RPHASE, SINGLE)
    | MCBSP_FMKS(RCR, RFRLEN2, DEFAULT)
    | MCBSP_FMKS(RCR, RWDLEN2, DEFAULT)
    | MCBSP_FMKS(RCR, RCOMPAND, MSB)
    | MCBSP_FMKS(RCR, RFIG, NO)
    | MCBSP_FMKS(RCR, RDATDLY, 1BIT);

   /*  Transmit Control Register                               */
   pAdc->serial->sConfig.xcr =
      MCBSP_FMKS(XCR, XFRLEN1, OF(0))
    | MCBSP_FMKS(XCR, XWDLEN1, 20BIT)
    | MCBSP_FMKS(XCR, XPHASE, SINGLE)
    | MCBSP_FMKS(XCR, XFRLEN2, DEFAULT)
    | MCBSP_FMKS(XCR, XWDLEN2, DEFAULT)
    | MCBSP_FMKS(XCR, XCOMPAND, MSB)
    | MCBSP_FMKS(XCR, XFIG, NO)
    | MCBSP_FMKS(XCR, XDATDLY, 1BIT);

   /*  Sample Rate Generator Register                          */
   pAdc->serial->sConfig.srgr =
      MCBSP_FMKS(SRGR, FWID, DEFAULT)
    | MCBSP_FMKS(SRGR, CLKGDV, OF(MCBSP0_CLKGDV_VALUE))
    | MCBSP_FMKS(SRGR, GSYNC, DEFAULT)
    | MCBSP_FMKS(SRGR, CLKSP, DEFAULT)
    | MCBSP_FMKS(SRGR, CLKSM, INTERNAL)
    | MCBSP_FMKS(SRGR, FSGM, DXR2XSR)
    | MCBSP_FMKS(SRGR, FPER, DEFAULT);

    /*  Pin Control Register                                    */
   pAdc->serial->sConfig.pcr =
      MCBSP_FMKS(PCR, XIOEN, SP)
    | MCBSP_FMKS(PCR, RIOEN, SP)
    | MCBSP_FMKS(PCR, FSXM, INTERNAL)
    | MCBSP_FMKS(PCR, FSRM, EXTERNAL)
    | MCBSP_FMKS(PCR, CLKXM, OUTPUT)
    | MCBSP_FMKS(PCR, CLKRM, INPUT)
    | MCBSP_FMKS(PCR, FSXP, ACTIVELOW)
    | MCBSP_FMKS(PCR, FSRP, ACTIVELOW)
    | MCBSP_FMKS(PCR, CLKXP, RISING)
    | MCBSP_FMKS(PCR, CLKRP, RISING);

   MCBSP_config(pAdc->serial->hMcbsp, &pAdc->serial->sConfig);
   /* start McBsp                                                */
   MCBSP_enableSrgr(pAdc->serial->hMcbsp);
   MCBSP_enableXmt(pAdc->serial->hMcbsp);
   MCBSP_enableRcv(pAdc->serial->hMcbsp);

   /* get receive event ID for specific serial port              */
   pAdc->serial->intnum = MCBSP_getRcvEventId(pAdc->serial->hMcbsp);
   
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* InitTimer()                                                    */
/**Operation:
 *    - inits and starts the selected timer
 *
 * Parameters:
 *    - Timer to use
 *    - Divider value for timer
 *    - Timer handle
 *
 * Return value:
 *    - TIDC_NO_ERR
 *    - TIDC_ERR_NOCHIPRES                                       */
/*****************************************************************/
static TTIDCSTATUS InitTimer(TADS8330 *pAdc)
{
   /* open Timer                                                 */
   pAdc->hTimer = TIMER_open(pAdc->iTimer, TIMER_OPEN_RESET);
   if (pAdc->hTimer == INV) {
      pAdc->hTimer = 0;
      return TIDC_ERR_NOCHIPRES;
   }

   /* if auto trigger mode: set DSP pin constantly high          */
   /* if CONVST pin mode  : timer toogles                        */
   if (pAdc->creg.control_bit.convstart == 0) {
      TIMER_configArgs(pAdc->hTimer,
                       TIMER_FMKS(CTL, INVINP, DEFAULT)
                      |TIMER_FMKS(CTL, CLKSRC, DEFAULT)
                      |TIMER_FMKS(CTL, CP, DEFAULT)
                      |TIMER_FMKS(CTL, HLD, DEFAULT)
                      |TIMER_FMKS(CTL, GO, DEFAULT)
                      |TIMER_FMKS(CTL, PWID, DEFAULT)
                      |TIMER_FMKS(CTL, DATOUT, 1)
                      |TIMER_FMKS(CTL, INVOUT, DEFAULT)
                      |TIMER_FMKS(CTL, FUNC, GPIO),
                      pAdc->uiTimDiv,
                      0x00000000);
   } else {
      TIMER_configArgs(pAdc->hTimer,
                       TIMER_FMKS(CTL, INVINP, DEFAULT)
#if C64_SUPPORT
                      |TIMER_FMKS(CTL, CLKSRC, CPUOVR8)
#else
                      |TIMER_FMKS(CTL, CLKSRC, CPUOVR4)
#endif
                      |TIMER_FMKS(CTL, CP, CLOCK)
                      |TIMER_FMKS(CTL, HLD, YES)
                      |TIMER_FMKS(CTL, GO, NO)
                      |TIMER_FMKS(CTL, PWID, DEFAULT)
                      |TIMER_FMKS(CTL, INVOUT, NO)
                      |TIMER_FMKS(CTL, FUNC, TOUT),
                      pAdc->uiTimDiv,
                      0x00000000);
   }
   /* start Timer                                                */
   TIMER_start(pAdc->hTimer);
   
   return TIDC_NO_ERR;
}

/*****************************************************************/
/* OpenEdma()                                                     */
/**Operation:
 *    - Opens the EDMA channel and two parameter sets
 *    - Configures the stop transfer
 *
 * Parameters:
 *    - Data converter object
 *
 * Return value:
 *    - TIDC_NO_ERR
 *    - TIDC_ERR_NOCHIPRES                                       */
/*****************************************************************/
static TTIDCSTATUS OpenEdma(TADS8330 *pAdc)
{
   /* open EDMA receive channel                                  */
   pAdc->hEdmaRx = EDMA_open(pAdc->serial->intnum, EDMA_OPEN_RESET);
   if (pAdc->hEdmaRx == EDMA_HINV) {
      pAdc->hEdmaRx = 0;
      return TIDC_ERR_NOCHIPRES;
   }
   /* open EDMA transmit channel                                 */
   pAdc->hEdmaTx = EDMA_open(pAdc->uiIntNum, EDMA_OPEN_RESET);
   if (pAdc->hEdmaTx == EDMA_HINV) {
      pAdc->hEdmaTx = 0;
      return TIDC_ERR_NOCHIPRES;
   }
   /* alloc parameter set "pend" for receive channel             */
   pAdc->hEdmaRx_pend = EDMA_allocTable(-1);
   if (pAdc->hEdmaRx_pend == EDMA_HINV) {
      pAdc->hEdmaRx_pend = 0;
      return TIDC_ERR_NOCHIPRES;
   }
   /* alloc parameter set "pend" for transmit channel            */
   pAdc->hEdmaTx_pend = EDMA_allocTable(-1);
   if (pAdc->hEdmaTx_pend == EDMA_HINV) {
      pAdc->hEdmaTx_pend = 0;
      return TIDC_ERR_NOCHIPRES;
   }
   /* alloc parameter set "stop"                                 */
   pAdc->hEdma_stop = EDMA_allocTable(-1);
   if (pAdc->hEdma_stop == EDMA_HINV) {
      pAdc->hEdma_stop = 0;
      return TIDC_ERR_NOCHIPRES;
   }
   /* configure "stop" parameter set 	                          */
   EDMA_configArgs(pAdc->hEdma_stop, 0, 0, 0, 0, 0, 0);
   /* enable EDMA CPU interrupt                                  */
   IRQ_enable(IRQ_EVT_EDMAINT);
   
   return TIDC_NO_ERR;
}

/****************************************************************/
/* END OF t8330_ob.c                                             */
/****************************************************************/
/*TIDC_Wizard Auto Code End}}*/
